<?php

namespace Service\Email;

use Util\Mail\Core\SmtpMail;
use Zend\Mail\Transport\Smtp as SmtpTransport;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;
use Zend\Mail\Message as ZendMessage;
use Zend\Mail\Transport\SmtpOptions;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class EmailPasswordSupportOperator
{
    public static function sendMail($cdoperador, $nmoperador, $to, $password, $client, $expirationDate)
    {
        $templateEmail = self::getTemplatePath();
        $content = self::decodeTemplateContent($templateEmail, $cdoperador, $to, $password, $client, $expirationDate);
        $subject = $nmoperador . ', sua senha foi criada.';
        self::sendHtml($to, 'sac@teknisa.com', $subject, $content);
    }

    private static function decodeTemplateContent($template, $code, $email, $password, $client, $expirationDate){
        $decodedContent = file_get_contents($template);
        $decodedContent = str_replace('%cliente%', $client, $decodedContent);
        $decodedContent = str_replace('%email%', $email, $decodedContent);
        $decodedContent = str_replace('%senha%', $password, $decodedContent);
        $decodedContent = str_replace('%data%', $expirationDate, $decodedContent);
        return $decodedContent;
    }

    private static function getTemplatePath()
    {
        return __DIR__.'/../../Util/EmailTemplates/generated_password.html';
    }

    private static function sendHtml($to, $from, $subject, $content)
    {
        try {
            $transport = new SmtpTransport();

            $mimePart = new ZendMimePart($content);
            $mimePart->type = 'text/html';

            $mimeMessage = new ZendMimeMessage();
            $mimeMessage->setParts(array($mimePart));

            $mensagem = new ZendMessage();
            $mensagem->setBody($mimeMessage)->addFrom($from)->addTo($to)->setSubject($subject);

            $transport->setOptions(self::getSmtpConfiguration());

            $transport->send($mensagem);
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    private static function getSmtpConfiguration(){
        $nome = InstanceManager::getInstance()->getParameter('smtpName');
        $host = InstanceManager::getInstance()->getParameter('smtpHost');
        $usuario = InstanceManager::getInstance()->getParameter('smtpUsername');
        $senha = InstanceManager::getInstance()->getParameter('smtpPassword');
        $port = InstanceManager::getInstance()->getParameter('smtpPort');
        $classeConexao = InstanceManager::getInstance()->getParameter('smtpConnectionClass');
        return new SmtpOptions(array(
            'name' => $nome,
            'host' => $host,
            'port' => $port,
            'connection_class' => $classeConexao,
            'connection_config' => array(
                'username' => $usuario,
                'password' => $senha
            )
        ));
    }
}
