<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;


use Util\Utilities;

class CalendarDetail extends \Doctrine\ORM\EntityRepository
{

    public function getCalendarException($calendar,$organization,$id = null, $filter = '')
    {
        $query = <<<SQL
        SELECT * FROM(
            SELECT CAD.ID,CAD.TYPE,CAD.STATUS,CAD.NAME,CAD.MONTH,CAD.EVERY AS EVERY_DAY,CAD.EVERY AS EVERY_MONTH_WEEK,
            CAD.EVERY AS EVERY_MONTH_YEAR,CAD.EVERY AS EVERY_WEEK ,CAD.TIME_BREAK,CAD.ORDER_DAY AS ORDER_DAY_MONTH,
            CAD.ORDER_DAY AS ORDER_DAY_YEAR,CAD.IS_EXCEPTION,CAD.RECURRENCE_UNIT AS RECURRENCE_TYPE,CAD.MONTH_DAY,
            TO_CHAR(CAD.END_DATE,'DD/MM/YYYY') AS END_DATE ,TO_CHAR(CAD.BEGIN_DATE,'DD/MM/YYYY') AS BEGIN_DATE,
            CAD.DAILY_HOURS AS WORKTIME,TIME_BREAK AS OCCURRENCES,(CASE WHEN CAD.TYPE = 'W' THEN 'Trabalho' ELSE 'Folga' END)
             AS TYPE_NAME,CAD.CALENDAR,CAD.ORGANIZATION,
            CAD.ORIGIN_CALENDAR_DETAIL
             FROM CALENDAR_DETAIL CAD
             WHERE CAD.IS_EXCEPTION = 'Y'
             AND CAD.ORGANIZATION = $organization
             AND CAD.CALENDAR = $calendar

SQL;
        if($id){
            $query .= <<<SQL
        AND CAD.ID = $id
SQL;
        }
        $query .= <<<SQL
         ORDER BY CAD.ID DESC
SQL;
        $filterString = ') WHERE 1 = 1 '. $filter;

        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query.$filterString);
        foreach($exceptions as &$exception ){
            $exception['WORKTIME'] = Utilities::secondsToTime($exception['WORKTIME']);
            if($exception['RECURRENCE_TYPE'] == \Service\CalendarDetail::RECURRENCE_TYPE_WEEKLY ){
                $weeks = $this->getAllWeekDaysByCalendarDetailId($exception['ID']);
                foreach($weeks as $week){
                    $exception[Utilities::$ARRAY_WEEK_BY_BD[$week['WEEK_DAY']]] = 'Y';
                }
            }else if($exception['RECURRENCE_TYPE'] == \Service\CalendarDetail::RECURRENCE_TYPE_YEARLY ){
                if($exception['MONTH']){
                    $monthName = Utilities::$ARRAY_NUMERIC_FOR_MONTH[$exception['MONTH']];
                    $exception['YEAR_DAY'] = $exception['MONTH_DAY'] . ' ' . $monthName;
                }
            }
        }
        return $exceptions;
    }
    public function getAllWeekDaysByCalendarDetailId($calendarDetailId){
        $query = <<<SQL
        SELECT * FROM CALENDAR_WEEKLY_DAYS WHERE CALENDAR_DETAIL = $calendarDetailId
SQL;
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query);
    }

    public function getOccurrencesForWeekRecurrence($beginDate, $endDate, $weekdaysString, $every)
    {
        $query = <<<SQL
        SELECT COUNT (*) AS OCCURRENCES FROM(
        SELECT *
          FROM TABLE (PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
           TO_DATE(:endDate,'DD/MM/YYYY HH24:MI:SS'))) DT
         WHERE DT.WEEK_DAY IN ($weekdaysString)
           AND MOD(DT.WEEK_IN_YEAR+:every-1, :every) = 0)

SQL;

        $param = array(
            'beginDate' => $beginDate,
            'endDate' => $endDate,
            'every' => $every
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions[0]['OCCURRENCES'];
    }

    public function getEndDateForWeekRecurrence($beginDate, $occurrences, $weekdaysString, $every)
    {
        $query = <<<SQL
        SELECT * FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS')+:occurrences*:every*7))
        WHERE WEEK_DAY IN ($weekdaysString))
SQL;

        //echo $query;die;
        $param = array(
            'beginDate' => $beginDate,
            'occurrences' => $occurrences,
            'every' => $every,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions;
    }

    public function getDate($beginDate)
    {
        $query = <<<SQL
       SELECT *
       FROM TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'), TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS')))

SQL;

        $param = array(
            'beginDate' => $beginDate,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions[0];
    }

    public function getOccurrencesForMonthlyDayRecurrence($beginDate, $endDate, $monthlyDay, $every, $month)
    {
        $query = <<<SQL
        SELECT COUNT (*) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         TO_DATE(:endDate,'DD/MM/YYYY HH24:MI:SS')))
         WHERE TO_CHAR(DATA,'DD') = $monthlyDay
         AND MOD(MONTH-(:month-:every),:every) = 0)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'endDate' => $endDate,
            'every' => $every,
            'month' => $month,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions[0]['OCCURRENCES'];
    }

    public function getOccurrencesForMonthlyWeekRecurrence($beginDate, $endDate, $orderDay, $weekDay, $every, $month)
    {
        if ($orderDay == 'L') {
        $filterOrderDay = ' AND WD_IS_LAST_IN_MONTH = 1';
    } else {
        $filterOrderDay = "AND WD_ORDER_IN_MONTH = $orderDay";
    }
        $query = <<<SQL
        SELECT COUNT (*) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         TO_DATE(:endDate,'DD/MM/YYYY HH24:MI:SS')))
         WHERE WEEK_DAY = :weekDay
          AND MOD(MONTH-(:month-:every),:every) = 0
         $filterOrderDay)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'endDate' => $endDate,
            'every' => $every,
            'orderDay' => $orderDay,
            'weekDay' => $weekDay,
            'month' => $month,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions[0]['OCCURRENCES'];
    }

    public function getEndDateForMonthlyDayRecurrence($beginDate, $occurrences, $monthlyDay, $every, $month)
    {
        $query = <<<SQL
        SELECT MAX(DATA) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         add_months(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),(:every*:occurrences))))
         WHERE TO_CHAR(DATA,'DD') = $monthlyDay
         AND MOD(MONTH-(:month-:every),:every) = 0)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'occurrences' => $occurrences,
            'every' => $every,
            'month' => $month,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT_BY_BD,$exceptions[0]['OCCURRENCES']);
    }

    public function getEndDateForMonthlyWeekRecurrence($beginDate, $occurrences, $orderDay, $weekDay, $every, $month)
    {
        if ($orderDay == 'L') {
            $filterOrderDay = ' AND WD_IS_LAST_IN_MONTH = 1';
        } else {
            $filterOrderDay = "AND WD_ORDER_IN_MONTH = $orderDay";
        }
        $query = <<<SQL
        SELECT MAX(DATA) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         add_months(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),(:every*:occurrences))))
         WHERE WEEK_DAY = :weekDay
          AND MOD(MONTH-(:month-:every),:every) = 0
         $filterOrderDay)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'occurrences' => $occurrences,
            'every' => $every,
            'orderDay' => $orderDay,
            'weekDay' => $weekDay,
            'month' => $month,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT_BY_BD,$exceptions[0]['OCCURRENCES']);
    }

    public function getOccurrencesForYearlyDayRecurrence($beginDate, $endDate, $monthlyDay, $monthly)
    {
        $query = <<<SQL
        SELECT COUNT (*) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         TO_DATE(:endDate,'DD/MM/YYYY HH24:MI:SS')))
         WHERE TO_CHAR(DATA,'DD') = :monthlyDay
         AND MONTH = :monthly)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'endDate' => $endDate,
            'monthlyDay' => $monthlyDay,
            'monthly' => $monthly,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions[0]['OCCURRENCES'];
    }

    public function getOccurrencesForYearlyWeekRecurrence($beginDate, $endDate, $orderDay, $weekDay, $monthly)
    {
        if ($orderDay == 'L') {
            $filterOrderDay = ' AND WD_IS_LAST_IN_MONTH = 1';
        } else {
            $filterOrderDay = "AND WD_ORDER_IN_MONTH = $orderDay";
        }
        $query = <<<SQL
        SELECT COUNT (*) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         TO_DATE(:endDate,'DD/MM/YYYY HH24:MI:SS')))
         WHERE WEEK_DAY = :weekDay
         AND MONTH = :monthly
         $filterOrderDay)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'endDate' => $endDate,
            'monthly' => $monthly,
            'orderDay' => $orderDay,
            'weekDay' => $weekDay,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return $exceptions[0]['OCCURRENCES'];
    }

    public function getEndDateForYearlyDayRecurrence($beginDate, $occurrences, $monthlyDay, $monthly)
    {
        $query = <<<SQL
        SELECT MAX(DATA)  AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         add_months(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),(12*:occurrences))))
         WHERE TO_CHAR(DATA,'DD') = :monthlyDay
         AND MONTH = :monthly
         AND ROWNUM <= :occurrences
         )
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'occurrences' => $occurrences,
            'monthlyDay' => $monthlyDay,
            'monthly' => $monthly,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT_BY_BD,$exceptions[0]['OCCURRENCES']);
    }

    public function getEndDateForYearlyWeekRecurrence($beginDate, $occurrences, $orderDay, $weekDay, $monthly)
    {
        if ($orderDay == 'L') {
            $filterOrderDay = ' AND WD_IS_LAST_IN_MONTH = 1';
        } else {
            $filterOrderDay = "AND WD_ORDER_IN_MONTH = $orderDay";
        }
        $query = <<<SQL
        SELECT MAX(DATA) AS OCCURRENCES FROM(
        SELECT * FROM
        TABLE(PM_DATE_WEEK_INFO(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),
         add_months(TO_DATE(:beginDate,'DD/MM/YYYY HH24:MI:SS'),(12*:occurrences))))
         WHERE WEEK_DAY = :weekDay
         AND MONTH = :monthly
         $filterOrderDay)
SQL;
        $param = array(
            'beginDate' => $beginDate,
            'occurrences' => $occurrences,
            'monthly' => $monthly,
            'orderDay' => $orderDay,
            'weekDay' => $weekDay,
        );
        $connection = $this->getEntityManager()->getConnection();
        $exceptions = $connection->fetchAll($query, $param);
        return \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT_BY_BD,$exceptions[0]['OCCURRENCES']);
    }
}